#!/usr/bin/env python3

# External command, intended to be called with meson.add_dist_script() in meson.build

#                        argv[1]          argv[2]
# extra-dist-cmd.py <root_source_dir> <root_build_dir>

# Meson does not preserve timestamps on distributed files.
# But this script preserves the timestamps on libstdc++.tag.

import os
import sys
import subprocess
import shutil

root_source_dir = sys.argv[1]
root_build_dir = sys.argv[2]

# Make a ChangeLog file for distribution.
cmd = [
  'git',
  '--git-dir=' + os.path.join(root_source_dir, '.git'),
  '--work-tree=' + root_source_dir,
  'log',
  '--no-merges',
  '--date=short',
  '--max-count=200',
  '--pretty=tformat:%cd  %an  <%ae>%n%n  %s%n%w(0,0,2)%+b',
]
with open(os.path.join(os.getenv('MESON_DIST_ROOT'), 'ChangeLog'), mode='w') as logfile:
  result = subprocess.run(cmd, stdout=logfile)

# Distribute the libstdc++.tag file in addition to the files in the local git clone.
# shutil.copy2() copies timestamps and some other file metadata.
shutil.copy2(os.path.join(root_build_dir, 'libstdc++.tag'),
             os.path.join(os.getenv('MESON_DIST_ROOT'), 'doctags'))

sys.exit(result.returncode)



#!/usr/bin/env python3

# External command, intended to be called with run_command() or custom_target()
# in meson.build

#                              argv[1]           argv[2]       argv[3:]
# skeletonmm-tarball.py <output_file_or_check> <source_dir> <input_files...>

import os
import sys
import shutil
import tarfile

if sys.argv[1] == 'check':
  # Called from run_command() during setup or configuration.
  # Check which archive format can be used.
  # In order from most wanted to least wanted: .tar.xz, .tar.gz, .tar
  available_archive_formats = []
  for af in shutil.get_archive_formats():
    # Keep the formats in a list, skip the descriptions.
    available_archive_formats += [af[0]]
  if 'xztar' in available_archive_formats:
    suffix = '.tar.xz'
  elif 'gztar' in available_archive_formats:
    suffix = '.tar.gz'
  else: # Uncompressed tar format is always available.
    suffix = '.tar'
  print(suffix, end='') # stdout can be read in the meson.build file.
  sys.exit(0)

# Create an archive.
output_file = sys.argv[1]
source_dir = sys.argv[2]

if output_file.endswith('.xz'):
  mode = 'w:xz'
elif output_file.endswith('.gz'):
  mode = 'w:gz'
else:
  mode = 'w'

with tarfile.open(output_file, mode=mode) as tar_file:
  os.chdir(source_dir) # Input filenames are relative to source_dir.
  for file in sys.argv[3:]:
    tar_file.add(file)
# Errors raise exceptions. If an exception is raised, Meson+ninja will notice
# that the command failed, despite exit(0).
sys.exit(0)

# shutil.make_archive() might be an alternative, but it only archives
# whole directories. It's not useful, if you want to have full control
# of which files are archived.



#!/usr/bin/env python3

# External command, intended to be called with custom_target() in meson.build

#                         argv[1]         argv[2]    argv[3]
# libstdcxx-tag.py <curl-or-wget-or-none> <srcdir> <output_path>

import os
import sys
import subprocess
import shutil

subcommand = sys.argv[1]
srcdir = sys.argv[2]
output_path = sys.argv[3]
output_dirname, output_filename = os.path.split(output_path)
if not output_dirname:
  output_dirname = '.'

# Remote location of the GNU libstdc++ Doxygen tag file.
libstdcxx_tag_url = 'http://gcc.gnu.org/onlinedocs/libstdc++/latest-doxygen/' + output_filename

def curl():
  cmd = [
    subcommand,
    '--compressed',
    '--connect-timeout', '300',
    '--globoff',
    '--location',
    '--max-time', '3600',
    '--remote-time',
    '--retry', '5',
  ]
  if os.path.isfile(output_path):
    # Don't download the tag file unless it's newer than the local file.
    cmd += ['--time-cond', output_path]

  cmd += [
    '--output', output_path,
    libstdcxx_tag_url,
  ]
  return subprocess.run(cmd).returncode

def wget():
  cmd = [
    subcommand,
    '--timestamping',
    '--no-directories',
    '--timeout=300',
    '--tries=5',
    '--directory-prefix=' + output_dirname,
      libstdcxx_tag_url,
    ]
  return subprocess.run(cmd).returncode

def dont_download_tag_file():
  if os.path.isfile(output_path):
    print('Did not check status of', output_path, 'because network is disabled.')
  elif os.path.isfile(os.path.join(srcdir, output_filename)):
    print('Warning:', output_path, 'does not exist.')
    print('Copying from the source directory because network is disabled.')
    print('If you want an up-to-date copy, reconfigure with the -Duse-network=true option.')
    # shutil.copy2() copies timestamps and some other file metadata.
    shutil.copy2(os.path.join(srcdir, output_filename), output_path)
  else:
    print('Error:', output_path, 'does not exist.', file=sys.stderr)
    print('Downloading it is not possible because network is disabled.', file=sys.stderr)
    print('Please reconfigure with the -Duse-network=true option.', file=sys.stderr)
    return 1
  return 0

# ----- Main -----
subcommand_base = os.path.splitext(os.path.basename(os.path.normpath(subcommand)))[0]
if subcommand_base == 'curl':
  sys.exit(curl())
if subcommand_base == 'wget':
  sys.exit(wget())
sys.exit(dont_download_tag_file())



#!/usr/bin/env python3

# External command, intended to be called with meson.add_install_script() in meson.build

#                           argv[1]
# extra-install-cmd.py <aclocal_macrodir>

import os
import sys
import subprocess

if not os.getenv('DESTDIR'):
  # Inform the installer that M4 macro files installed in a directory
  # not known to aclocal will not be picked up automatically.
  # (Starting with Python 3.7 text=True is a more understandable equivalent to
  # universal_newlines=True. Let's use only features in Python 3.5.)
  result = subprocess.run(['aclocal', '--print-ac-dir'],
                          stdout=subprocess.PIPE, stderr=subprocess.DEVNULL,
                          universal_newlines=True)
  acdir = result.stdout
  aclocal_path = os.getenv('ACLOCAL_PATH')
  # acdir and aclocal_path can be sequences of os.pathsep-separated paths.
  # Merge them to one sequence with leading and trailing os.pathsep.
  # os.pathsep is ':' for Linux, ';' for Windows.
  acdirs = os.pathsep
  if aclocal_path:
    acdirs += aclocal_path + os.pathsep
  if acdir:
    acdirs += acdir + os.pathsep

  if (os.pathsep + sys.argv[1] + os.pathsep) not in acdirs:
    # f'''.....''' would require Python 3.6. Avoid it.
    print('''\
                NOTE
                ----
The mm-common Autoconf macro files have been installed in a different
directory than the system aclocal directory. In order for the installed
macros to be found, it may be necessary to add the mm-common include
path to the ACLOCAL_PATH environment variable:
  ACLOCAL_PATH="$ACLOCAL_PATH:{}"
  export ACLOCAL_PATH'''.format(sys.argv[1])
    )
sys.exit(0)
